@@ -22,11 +22,14 @@ from abc import ABC, abstractmethod
 from typing import List
 
 import cloudpickle
+import avro.schema as avro_schema
 
 from pyflink.common import Row, RowKind
 from pyflink.common.time import Instant
 from pyflink.datastream.window import TimeWindow, CountWindow
 from pyflink.fn_execution.ResettableIO import ResettableIO
+from pyflink.fn_execution.formats.avro import FlinkAvroDecoder, FlinkAvroDatumReader, \
+    FlinkAvroBufferWrapper, FlinkAvroEncoder, FlinkAvroDatumWriter
 from pyflink.fn_execution.stream_slow import InputStream, OutputStream
 from pyflink.table.utils import pandas_to_arrow, arrow_to_pandas
 
@@ -800,6 +803,19 @@ class CountWindowCoderImpl(FieldCoderImpl):
         return CountWindow(in_stream.read_int64())
 
 
+class GlobalWindowCoderImpl(FieldCoderImpl):
+    """
+    A coder for CountWindow.
+    """
+
+    def encode_to_stream(self, value, out_stream: OutputStream):
+        out_stream.write_byte(0)
+
+    def decode_from_stream(self, in_stream: InputStream, length=0):
+        in_stream.read_byte()
+        return GlobalWindowCoderImpl()
+
+
 class DataViewFilterCoderImpl(FieldCoderImpl):
     """
     A coder for data view filter.
@@ -823,6 +839,26 @@ class DataViewFilterCoderImpl(FieldCoderImpl):
         return row
 
 
+class AvroCoderImpl(FieldCoderImpl):
+
+    def __init__(self, schema_string: str):
+        self._buffer_wrapper = FlinkAvroBufferWrapper()
+        self._schema = avro_schema.parse(schema_string)
+        self._decoder = FlinkAvroDecoder(self._buffer_wrapper)
+        self._encoder = FlinkAvroEncoder(self._buffer_wrapper)
+        self._reader = FlinkAvroDatumReader(writer_schema=self._schema, reader_schema=self._schema)
+        self._writer = FlinkAvroDatumWriter(writer_schema=self._schema)
+
+    def encode_to_stream(self, value, out_stream: OutputStream):
+        self._buffer_wrapper.switch_stream(out_stream)
+        self._writer.write(value, self._encoder)
+
+    def decode_from_stream(self, in_stream: InputStream, length: int = 0):
+        # Since writer_schema equals reader_schema, in_stream does not need to support seek and tell
+        self._buffer_wrapper.switch_stream(in_stream)
+        return self._reader.read(self._decoder)
+
+
 class LocalDateCoderImpl(FieldCoderImpl):
 
     @staticmethod
