@@ -23,7 +23,8 @@ from typing import Tuple
 from pyflink.fn_execution.coders import DataViewFilterCoder, PickleCoder
 from pyflink.fn_execution.datastream.timerservice import InternalTimer
 from pyflink.fn_execution.datastream.operations import Operation
-from pyflink.fn_execution.datastream.timerservice_impl import TimerOperandType, InternalTimerImpl
+from pyflink.fn_execution.datastream.process.timerservice_impl import (
+    TimerOperandType, InternalTimerImpl)
 from pyflink.fn_execution.table.state_data_view import extract_data_view_specs
 
 from pyflink.fn_execution.table.window_assigner import TumblingWindowAssigner, \
@@ -33,6 +34,7 @@ from pyflink.fn_execution.table.window_trigger import EventTimeTrigger, Processi
     CountTrigger
 from pyflink.fn_execution.utils import operation_utils
 from pyflink.fn_execution.utils.operation_utils import extract_user_defined_aggregate_function
+from pyflink.fn_execution.metrics.process.metric_impl import GenericMetricGroup
 
 try:
     from pyflink.fn_execution.table.aggregate_fast import RowKeySelector, \
@@ -76,9 +78,24 @@ class BundleOperation(object):
 
 class BaseOperation(Operation):
     def __init__(self, serialized_fn):
-        super(BaseOperation, self).__init__(serialized_fn)
+        if serialized_fn.metric_enabled:
+            self.base_metric_group = GenericMetricGroup(None, None)
+        else:
+            self.base_metric_group = None
         self.func, self.user_defined_funcs = self.generate_func(serialized_fn)
 
+    def finish(self):
+        self._update_gauge(self.base_metric_group)
+
+    def _update_gauge(self, base_metric_group):
+        if base_metric_group is not None:
+            for name in base_metric_group._flink_gauge:
+                flink_gauge = base_metric_group._flink_gauge[name]
+                beam_gauge = base_metric_group._beam_gauge[name]
+                beam_gauge.set(flink_gauge())
+            for sub_group in base_metric_group._sub_groups:
+                self._update_gauge(sub_group)
+
     def process_element(self, value):
         return self.func(value)
 
@@ -180,9 +197,9 @@ class PandasBatchOverWindowAggregateFunctionOperation(BaseOperation):
         bounded_range_window_nums = 0
         for i, window in enumerate(self.windows):
             window_type = window.window_type
-            if (window_type == window_types.RANGE_UNBOUNDED_PRECEDING) or (
-                    window_type == window_types.RANGE_UNBOUNDED_FOLLOWING) or (
-                    window_type == window_types.RANGE_SLIDING):
+            if (window_type is window_types.RANGE_UNBOUNDED_PRECEDING) or (
+                    window_type is window_types.RANGE_UNBOUNDED_FOLLOWING) or (
+                    window_type is window_types.RANGE_SLIDING):
                 self.bounded_range_window_index[i] = bounded_range_window_nums
                 self.is_bounded_range_window.append(True)
                 bounded_range_window_nums += 1
@@ -221,13 +238,13 @@ class PandasBatchOverWindowAggregateFunctionOperation(BaseOperation):
             if self.is_bounded_range_window[window_index]:
                 window_boundaries = boundaries_series[
                     self.bounded_range_window_index[window_index]]
-                if window_type == OverWindow.RANGE_UNBOUNDED_PRECEDING:
+                if window_type is OverWindow.RANGE_UNBOUNDED_PRECEDING:
                     # range unbounded preceding window
                     for j in range(input_cnt):
                         end = window_boundaries[j]
                         series_slices = [s.iloc[:end] for s in input_series]
                         result.append(func(series_slices))
-                elif window_type == OverWindow.RANGE_UNBOUNDED_FOLLOWING:
+                elif window_type is OverWindow.RANGE_UNBOUNDED_FOLLOWING:
                     # range unbounded following window
                     for j in range(input_cnt):
                         start = window_boundaries[j]
@@ -242,19 +259,19 @@ class PandasBatchOverWindowAggregateFunctionOperation(BaseOperation):
                         result.append(func(series_slices))
             else:
                 # unbounded range window or unbounded row window
-                if (window_type == OverWindow.RANGE_UNBOUNDED) or (
-                        window_type == OverWindow.ROW_UNBOUNDED):
+                if (window_type is OverWindow.RANGE_UNBOUNDED) or (
+                        window_type is OverWindow.ROW_UNBOUNDED):
                     series_slices = [s.iloc[:] for s in input_series]
                     func_result = func(series_slices)
                     result = [func_result for _ in range(input_cnt)]
-                elif window_type == OverWindow.ROW_UNBOUNDED_PRECEDING:
+                elif window_type is OverWindow.ROW_UNBOUNDED_PRECEDING:
                     # row unbounded preceding window
                     window_end = window.upper_boundary
                     for j in range(input_cnt):
                         end = min(j + window_end + 1, input_cnt)
                         series_slices = [s.iloc[: end] for s in input_series]
                         result.append(func(series_slices))
-                elif window_type == OverWindow.ROW_UNBOUNDED_FOLLOWING:
+                elif window_type is OverWindow.ROW_UNBOUNDED_FOLLOWING:
                     # row unbounded following window
                     window_start = window.lower_boundary
                     for j in range(input_cnt):
