@@ -19,17 +19,20 @@ from typing import Union
 
 from pyflink import add_version_doc
 from pyflink.java_gateway import get_gateway
-from pyflink.table.expression import Expression, _get_java_expression, TimePointUnit
-from pyflink.table.types import _to_java_data_type, DataType, _to_java_type
-from pyflink.table.udf import UserDefinedFunctionWrapper, UserDefinedTableFunctionWrapper
+from pyflink.table.expression import Expression, _get_java_expression, TimePointUnit, JsonOnNull
+from pyflink.table.types import _to_java_data_type, DataType
+from pyflink.table.udf import UserDefinedFunctionWrapper
 from pyflink.util.java_utils import to_jarray, load_java_class
 
-__all__ = ['if_then_else', 'lit', 'col', 'range_', 'and_', 'or_', 'UNBOUNDED_ROW',
-           'UNBOUNDED_RANGE', 'CURRENT_ROW', 'CURRENT_RANGE', 'current_date', 'current_time',
-           'current_timestamp', 'local_time', 'local_timestamp', 'temporal_overlaps',
-           'date_format', 'timestamp_diff', 'array', 'row', 'map_', 'row_interval', 'pi', 'e',
-           'rand', 'rand_integer', 'atan2', 'negative', 'concat', 'concat_ws', 'uuid', 'null_of',
-           'log', 'with_columns', 'without_columns', 'call', 'call_sql']
+__all__ = ['if_then_else', 'lit', 'col', 'range_', 'and_', 'or_', 'not_', 'UNBOUNDED_ROW',
+           'UNBOUNDED_RANGE', 'CURRENT_ROW', 'CURRENT_RANGE', 'current_database',
+           'current_date', 'current_time', 'current_timestamp',
+           'current_watermark', 'local_time', 'local_timestamp',
+           'temporal_overlaps', 'date_format', 'timestamp_diff', 'array', 'row', 'map_',
+           'row_interval', 'pi', 'e', 'rand', 'rand_integer', 'atan2', 'negative', 'concat',
+           'concat_ws', 'uuid', 'null_of', 'log', 'with_columns', 'without_columns', 'json_string',
+           'json_object', 'json_object_agg', 'json_array', 'json_array_agg', 'call', 'call_sql',
+           'source_watermark']
 
 
 def _leaf_op(op_name: str) -> Expression:
@@ -66,6 +69,12 @@ def _quaternion_op(op_name: str, first, second, third, forth) -> Expression:
         _get_java_expression(forth)))
 
 
+def _varargs_op(op_name: str, *args):
+    gateway = get_gateway()
+    return Expression(
+        getattr(gateway.jvm.Expressions, op_name)(*[_get_java_expression(arg) for arg in args]))
+
+
 def _add_version_doc():
     from inspect import getmembers, isfunction
     from pyflink.table import expressions
@@ -76,7 +85,7 @@ def _add_version_doc():
 
 def col(name: str) -> Expression:
     """
-    Creates an expression which refers to a table's field.
+    Creates an expression which refers to a table's column.
 
     Example:
     ::
@@ -85,7 +94,7 @@ def col(name: str) -> Expression:
 
     :param name: the field name to refer to
     """
-    return _unary_op("$", name)
+    return _unary_op("col", name)
 
 
 def lit(v, data_type: DataType = None) -> Expression:
@@ -142,6 +151,25 @@ def or_(predicate0: Union[bool, Expression[bool]],
     return _ternary_op("or", predicate0, predicate1, predicates)
 
 
+def not_(expression: Expression[bool]) -> Expression[bool]:
+    """
+    Inverts a given boolean expression.
+
+    This method supports a three-valued logic by preserving `NULL`. This means if the input
+    expression is `NULL`, the result will also be `NULL`.
+
+    The resulting type is nullable if and only if the input type is nullable.
+
+    Examples:
+    ::
+
+        >>> not_(lit(True)) # False
+        >>> not_(lit(False)) # True
+        >>> not_(lit(None, DataTypes.BOOLEAN())) # None
+    """
+    return _unary_op("not", expression)
+
+
 """
 Offset constant to be used in the `preceding` clause of unbounded
 :class:`~pyflink.table.window.Over`. Use this constant for a time interval.
@@ -181,6 +209,13 @@ all rows with the same sort key as the current row are included in the window.
 CURRENT_RANGE = Expression("CURRENT_RANGE")  # type: Expression
 
 
+def current_database() -> Expression:
+    """
+    Returns the current database
+    """
+    return _leaf_op("currentDatabase")
+
+
 def current_date() -> Expression:
     """
     Returns the current SQL date in local time zone.
@@ -203,6 +238,25 @@ def current_timestamp() -> Expression:
     return _leaf_op("currentTimestamp")
 
 
+def current_watermark(rowtimeAttribute) -> Expression:
+    """
+    Returns the current watermark for the given rowtime attribute, or NULL if no common watermark of
+    all upstream operations is available at the current operation in the pipeline.
+
+    The function returns the watermark with the same type as the rowtime attribute, but with an
+    adjusted precision of 3. For example, if the rowtime attribute is `TIMESTAMP_LTZ(9)`, the
+    function will return `TIMESTAMP_LTZ(3)`.
+
+    If no watermark has been emitted yet, the function will return `NULL`. Users must take care of
+    this when comparing against it, e.g. in order to filter out late data you can use
+
+    ::
+
+        WHERE CURRENT_WATERMARK(ts) IS NULL OR ts > CURRENT_WATERMARK(ts)
+    """
+    return _unary_op("currentWatermark", rowtimeAttribute)
+
+
 def local_time() -> Expression:
     """
     Returns the current SQL time in local time zone.
@@ -293,6 +347,17 @@ def timestamp_diff(time_point_unit: TimePointUnit, time_point1, time_point2) ->
                        time_point1, time_point2)
 
 
+def from_unixtime(unixtime, format=None) -> Expression:
+    """
+    Convert unix timestamp (seconds since '1970-01-01 00:00:00' UTC) to datetime string the given
+    format. The default format is "yyyy-MM-dd HH:mm:ss".
+    """
+    if format is None:
+        return _unary_op("fromUnixtime", unixtime)
+    else:
+        return _binary_op("fromUnixtime", unixtime, format)
+
+
 def array(head, *tail) -> Expression:
     """
     Creates an array of literals.
@@ -392,7 +457,7 @@ def rand(seed: Union[int, Expression[int]] = None) -> Expression[float]:
 def rand_integer(bound: Union[int, Expression[int]],
                  seed: Union[int, Expression[int]] = None) -> Expression:
     """
-    Returns a pseudorandom integer value between 0.0 (inclusive) and the specified value
+    Returns a pseudorandom integer value between 0 (inclusive) and the specified value
     (exclusive) with a initial seed if specified. Two rand_integer() functions will return
     identical sequences of numbers if they have same initial seed and same bound.
     """
@@ -505,6 +570,29 @@ def if_then_else(condition: Union[bool, Expression[bool]], if_true, if_false) ->
     return _ternary_op("ifThenElse", condition, if_true, if_false)
 
 
+def coalesce(*args) -> Expression:
+    """
+    Returns the first argument that is not NULL.
+
+    If all arguments are NULL, it returns NULL as well.
+    The return type is the least restrictive, common type of all of its arguments.
+    The return type is nullable if all arguments are nullable as well.
+
+    Examples:
+    ::
+
+        >>> coalesce(None, "default") # Returns "default"
+        >>> # Returns the first non-null value among f0 and f1,
+        >>> # or "default" if f0 and f1 are both null
+        >>> coalesce(col("f0"), col("f1"), "default")
+
+    :param args: the input expressions.
+    """
+    gateway = get_gateway()
+    args = to_jarray(gateway.jvm.Object, [_get_java_expression(arg) for arg in args])
+    return _unary_op("coalesce", args)
+
+
 def with_columns(head, *tails) -> Expression:
     """
     Creates an expression that selects a range of columns. It can be used wherever an array of
@@ -542,6 +630,131 @@ def without_columns(head, *tails) -> Expression:
     return _binary_op("withoutColumns", head, tails)
 
 
+def json_string(value) -> Expression:
+    """
+    Serializes a value into JSON.
+
+    This function returns a JSON string containing the serialized value. If the value is `NULL`,
+    the function returns `NULL`.
+
+    Examples:
+    ::
+
+        >>> json_string(null_of(DataTypes.INT())) # None
+
+        >>> json_string(1)               # '1'
+        >>> json_string(True)            # 'true'
+        >>> json_string("Hello, World!") # '"Hello, World!"'
+        >>> json_string([1, 2])          # '[1,2]'
+    """
+    return _unary_op("jsonString", value)
+
+
+def json_object(on_null: JsonOnNull = JsonOnNull.NULL, *args) -> Expression:
+    """
+    Builds a JSON object string from a list of key-value pairs.
+
+    `args` is an even-numbered list of alternating key/value pairs. Note that keys must be
+    non-`NULL` string literals, while values may be arbitrary expressions.
+
+    This function returns a JSON string. The `on_null` behavior defines how to treat `NULL` values.
+
+    Values which are created from another JSON construction function call (`json_object`,
+    `json_array`) are inserted directly rather than as a string. This allows building nested JSON
+    structures.
+
+    Examples:
+    ::
+
+        >>> json_object() # '{}'
+        >>> json_object(JsonOnNull.NULL, "K1", "V1", "K2", "V2") # '{"K1":"V1","K2":"V2"}'
+
+        >>> # Expressions as values
+        >>> json_object(JsonOnNull.NULL, "orderNo", col("orderId"))
+
+        >>> json_object(JsonOnNull.NULL, "K1", null_of(DataTypes.STRING()))   # '{"K1":null}'
+        >>> json_object(JsonOnNull.ABSENT, "K1", null_of(DataTypes.STRING())) # '{}'
+
+        >>> # '{"K1":{"K2":"V"}}'
+        >>> json_object(JsonOnNull.NULL, "K1", json_object(JsonOnNull.NULL, "K2", "V"))
+
+    .. seealso:: :func:`~pyflink.table.expressions.json_array`
+    """
+    return _varargs_op("jsonObject", *(on_null._to_j_json_on_null(), *args))
+
+
+def json_object_agg(on_null: JsonOnNull,
+                    key_expr: Union[str, Expression[str]],
+                    value_expr) -> Expression:
+    """
+    Builds a JSON object string by aggregating key-value expressions into a single JSON object.
+
+    The key expression must return a non-nullable character string. Value expressions can be
+    arbitrary, including other JSON functions. If a value is `NULL`, the `on_null` behavior defines
+    what to do.
+
+    Note that keys must be unique. If a key occurs multiple times, an error will be thrown.
+
+    This function is currently not supported in `OVER` windows.
+
+    Examples:
+    ::
+
+        >>> # '{"Apple":2,"Banana":17,"Orange":0}'
+        >>> orders.select(json_object_agg(JsonOnNull.NULL, col("product"), col("cnt")))
+    """
+    return _ternary_op("jsonObjectAgg", on_null._to_j_json_on_null(), key_expr, value_expr)
+
+
+def json_array(on_null: JsonOnNull = JsonOnNull.ABSENT, *args) -> Expression:
+    """
+    Builds a JSON array string from a list of values.
+
+    This function returns a JSON string. The values can be arbitrary expressions. The `on_null`
+    behavior defines how to treat `NULL` values.
+
+    Elements which are created from another JSON construction function call (`json_object`,
+    `json_array`) are inserted directly rather than as a string. This allows building nested JSON
+    structures.
+
+    Examples:
+    ::
+
+        >>> json_array() # '[]'
+        >>> json_array(JsonOnNull.NULL, 1, "2") # '[1,"2"]'
+
+        >>> # Expressions as values
+        >>> json_array(JsonOnNull.NULL, col("orderId"))
+
+        >>> json_array(JsonOnNull.NULL, null_of(DataTypes.STRING()))   # '[null]'
+        >>> json_array(JsonOnNull.ABSENT, null_of(DataTypes.STRING())) # '[]'
+
+        >>> json_array(JsonOnNull.NULL, json_array(JsonOnNull.NULL, 1)) # '[[1]]'
+
+    .. seealso:: :func:`~pyflink.table.expressions.json_object`
+    """
+    return _varargs_op("jsonArray", *(on_null._to_j_json_on_null(), *args))
+
+
+def json_array_agg(on_null: JsonOnNull, item_expr) -> Expression:
+    """
+    Builds a JSON object string by aggregating items into an array.
+
+    Item expressions can be arbitrary, including other JSON functions. If a value is `NULL`, the
+    `on_null` behavior defines what to do.
+
+    This function is currently not supported in `OVER` windows, unbounded session windows, or hop
+    windows.
+
+    Examples:
+    ::
+
+        >>> # '["Apple","Banana","Orange"]'
+        >>> orders.select(json_array_agg(JsonOnNull.NULL, col("product")))
+    """
+    return _binary_op("jsonArrayAgg", on_null._to_j_json_on_null(), item_expr)
+
+
 def call(f: Union[str, UserDefinedFunctionWrapper], *args) -> Expression:
     """
     The first parameter `f` could be a str or a Python user-defined function.
@@ -573,21 +786,6 @@ def call(f: Union[str, UserDefinedFunctionWrapper], *args) -> Expression:
         return Expression(gateway.jvm.Expressions.call(
             f, to_jarray(gateway.jvm.Object, [_get_java_expression(arg) for arg in args])))
 
-    def get_function_definition(f):
-        if isinstance(f, UserDefinedTableFunctionWrapper):
-            """
-            TypeInference was not supported for TableFunction in the old planner. Use
-            TableFunctionDefinition to work around this issue.
-            """
-            j_result_types = to_jarray(gateway.jvm.TypeInformation,
-                                       [_to_java_type(i) for i in f._result_types])
-            j_result_type = gateway.jvm.org.apache.flink.api.java.typeutils.RowTypeInfo(
-                j_result_types)
-            return gateway.jvm.org.apache.flink.table.functions.TableFunctionDefinition(
-                'f', f._java_user_defined_function(), j_result_type)
-        else:
-            return f._java_user_defined_function()
-
     expressions_clz = load_java_class("org.apache.flink.table.api.Expressions")
     function_definition_clz = load_java_class('org.apache.flink.table.functions.FunctionDefinition')
     j_object_array_type = to_jarray(gateway.jvm.Object, []).getClass()
@@ -600,7 +798,7 @@ def call(f: Union[str, UserDefinedFunctionWrapper], *args) -> Expression:
     return Expression(api_call_method.invoke(
         None,
         to_jarray(gateway.jvm.Object,
-                  [get_function_definition(f),
+                  [f._java_user_defined_function(),
                    to_jarray(gateway.jvm.Object, [_get_java_expression(arg) for arg in args])])))
 
 
