@@ -114,7 +114,7 @@ class Table(object):
                 % (name, ', '.join(self.get_schema().get_field_names())))
         return col(name)
 
-    def select(self, *fields: Union[str, Expression]) -> 'Table':
+    def select(self, *fields: Expression) -> 'Table':
         """
         Performs a selection operation. Similar to a SQL SELECT statement. The field expressions
         can contain complex expressions.
@@ -128,12 +128,7 @@ class Table(object):
 
         :return: The result table.
         """
-        if all(isinstance(f, Expression) for f in fields):
-            return Table(self._j_table.select(to_expression_jarray(fields)), self._t_env)
-        else:
-            assert len(fields) == 1
-            assert isinstance(fields[0], str)
-            return Table(self._j_table.select(fields[0]), self._t_env)
+        return Table(self._j_table.select(to_expression_jarray(fields)), self._t_env)
 
     def alias(self, field: str, *fields: str) -> 'Table':
         """
@@ -153,7 +148,7 @@ class Table(object):
         extra_fields = to_jarray(gateway.jvm.String, fields)
         return Table(get_method(self._j_table, "as")(field, extra_fields), self._t_env)
 
-    def filter(self, predicate: Union[str, Expression[bool]]) -> 'Table':
+    def filter(self, predicate: Expression[bool]) -> 'Table':
         """
         Filters out elements that don't pass the filter predicate. Similar to a SQL WHERE
         clause.
@@ -168,7 +163,7 @@ class Table(object):
         """
         return Table(self._j_table.filter(_get_java_expression(predicate)), self._t_env)
 
-    def where(self, predicate: Union[str, Expression[bool]]) -> 'Table':
+    def where(self, predicate: Expression[bool]) -> 'Table':
         """
         Filters out elements that don't pass the filter predicate. Similar to a SQL WHERE
         clause.
@@ -183,7 +178,7 @@ class Table(object):
         """
         return Table(self._j_table.where(_get_java_expression(predicate)), self._t_env)
 
-    def group_by(self, *fields: Union[str, Expression]) -> 'GroupedTable':
+    def group_by(self, *fields: Expression) -> 'GroupedTable':
         """
         Groups the elements on some grouping keys. Use this before a selection with aggregations
         to perform the aggregation on a per-group basis. Similar to a SQL GROUP BY statement.
@@ -196,12 +191,7 @@ class Table(object):
         :param fields: Group keys.
         :return: The grouped table.
         """
-        if all(isinstance(f, Expression) for f in fields):
-            return GroupedTable(self._j_table.groupBy(to_expression_jarray(fields)), self._t_env)
-        else:
-            assert len(fields) == 1
-            assert isinstance(fields[0], str)
-            return GroupedTable(self._j_table.groupBy(fields[0]), self._t_env)
+        return GroupedTable(self._j_table.groupBy(to_expression_jarray(fields)), self._t_env)
 
     def distinct(self) -> 'Table':
         """
@@ -216,7 +206,7 @@ class Table(object):
         """
         return Table(self._j_table.distinct(), self._t_env)
 
-    def join(self, right: 'Table', join_predicate: Union[str, Expression[bool]] = None):
+    def join(self, right: 'Table', join_predicate: Expression[bool] = None):
         """
         Joins two :class:`~pyflink.table.Table`. Similar to a SQL join. The fields of the two joined
         operations must not overlap, use :func:`~pyflink.table.Table.alias` to rename fields if
@@ -245,7 +235,7 @@ class Table(object):
 
     def left_outer_join(self,
                         right: 'Table',
-                        join_predicate: Union[str, Expression[bool]] = None) -> 'Table':
+                        join_predicate: Expression[bool] = None) -> 'Table':
         """
         Joins two :class:`~pyflink.table.Table`. Similar to a SQL left outer join. The fields of
         the two joined operations must not overlap, use :func:`~pyflink.table.Table.alias` to
@@ -274,7 +264,7 @@ class Table(object):
 
     def right_outer_join(self,
                          right: 'Table',
-                         join_predicate: Union[str, Expression[bool]]) -> 'Table':
+                         join_predicate: Expression[bool]) -> 'Table':
         """
         Joins two :class:`~pyflink.table.Table`. Similar to a SQL right outer join. The fields of
         the two joined operations must not overlap, use :func:`~pyflink.table.Table.alias` to
@@ -299,7 +289,7 @@ class Table(object):
 
     def full_outer_join(self,
                         right: 'Table',
-                        join_predicate: Union[str, Expression[bool]]) -> 'Table':
+                        join_predicate: Expression[bool]) -> 'Table':
         """
         Joins two :class:`~pyflink.table.Table`. Similar to a SQL full outer join. The fields of
         the two joined operations must not overlap, use :func:`~pyflink.table.Table.alias` to
@@ -323,8 +313,8 @@ class Table(object):
             right._j_table, _get_java_expression(join_predicate)), self._t_env)
 
     def join_lateral(self,
-                     table_function_call: Union[str, Expression, UserDefinedTableFunctionWrapper],
-                     join_predicate: Union[str, Expression[bool]] = None) -> 'Table':
+                     table_function_call: Union[Expression, UserDefinedTableFunctionWrapper],
+                     join_predicate: Expression[bool] = None) -> 'Table':
         """
         Joins this Table with an user-defined TableFunction. This join is similar to a SQL inner
         join but works with a table function. Each row of the table is joined with the rows
@@ -368,9 +358,9 @@ class Table(object):
                 self._t_env)
 
     def left_outer_join_lateral(self,
-                                table_function_call: Union[str, Expression,
+                                table_function_call: Union[Expression,
                                                            UserDefinedTableFunctionWrapper],
-                                join_predicate: Union[str, Expression[bool]] = None) -> 'Table':
+                                join_predicate: Expression[bool] = None) -> 'Table':
         """
         Joins this Table with an user-defined TableFunction. This join is similar to
         a SQL left outer join but works with a table function. Each row of the table is joined
@@ -540,7 +530,7 @@ class Table(object):
         """
         return Table(self._j_table.intersectAll(right._j_table), self._t_env)
 
-    def order_by(self, *fields: Union[str, Expression]) -> 'Table':
+    def order_by(self, *fields: Expression) -> 'Table':
         """
         Sorts the given :class:`~pyflink.table.Table`. Similar to SQL ORDER BY.
         The resulting Table is sorted globally sorted across all parallel partitions.
@@ -556,12 +546,7 @@ class Table(object):
         :param fields: Order fields expression string.
         :return: The result table.
         """
-        if all(isinstance(f, Expression) for f in fields):
-            return Table(self._j_table.orderBy(to_expression_jarray(fields)), self._t_env)
-        else:
-            assert len(fields) == 1
-            assert isinstance(fields[0], str)
-            return Table(self._j_table.orderBy(fields[0]), self._t_env)
+        return Table(self._j_table.orderBy(to_expression_jarray(fields)), self._t_env)
 
     def offset(self, offset: int) -> 'Table':
         """
@@ -708,7 +693,7 @@ class Table(object):
                                  [item._java_over_window for item in over_windows])
         return OverWindowedTable(self._j_table.window(window_array), self._t_env)
 
-    def add_columns(self, *fields: Union[str, Expression]) -> 'Table':
+    def add_columns(self, *fields: Expression) -> 'Table':
         """
         Adds additional columns. Similar to a SQL SELECT statement. The field expressions
         can contain complex expressions, but can not contain aggregations. It will throw an
@@ -723,14 +708,9 @@ class Table(object):
         :param fields: Column list string.
         :return: The result table.
         """
-        if all(isinstance(f, Expression) for f in fields):
-            return Table(self._j_table.addColumns(to_expression_jarray(fields)), self._t_env)
-        else:
-            assert len(fields) == 1
-            assert isinstance(fields[0], str)
-            return Table(self._j_table.addColumns(fields[0]), self._t_env)
+        return Table(self._j_table.addColumns(to_expression_jarray(fields)), self._t_env)
 
-    def add_or_replace_columns(self, *fields: Union[str, Expression]) -> 'Table':
+    def add_or_replace_columns(self, *fields: Expression) -> 'Table':
         """
         Adds additional columns. Similar to a SQL SELECT statement. The field expressions
         can contain complex expressions, but can not contain aggregations. Existing fields will be
@@ -747,15 +727,10 @@ class Table(object):
         :param fields: Column list string.
         :return: The result table.
         """
-        if all(isinstance(f, Expression) for f in fields):
-            return Table(self._j_table.addOrReplaceColumns(to_expression_jarray(fields)),
-                         self._t_env)
-        else:
-            assert len(fields) == 1
-            assert isinstance(fields[0], str)
-            return Table(self._j_table.addOrReplaceColumns(fields[0]), self._t_env)
+        return Table(self._j_table.addOrReplaceColumns(to_expression_jarray(fields)),
+                     self._t_env)
 
-    def rename_columns(self, *fields: Union[str, Expression]) -> 'Table':
+    def rename_columns(self, *fields: Expression) -> 'Table':
         """
         Renames existing columns. Similar to a field alias statement. The field expressions
         should be alias expressions, and only the existing fields can be renamed.
@@ -768,15 +743,10 @@ class Table(object):
         :param fields: Column list string.
         :return: The result table.
         """
-        if all(isinstance(f, Expression) for f in fields):
-            return Table(self._j_table.renameColumns(to_expression_jarray(fields)),
-                         self._t_env)
-        else:
-            assert len(fields) == 1
-            assert isinstance(fields[0], str)
-            return Table(self._j_table.renameColumns(fields[0]), self._t_env)
+        return Table(self._j_table.renameColumns(to_expression_jarray(fields)),
+                     self._t_env)
 
-    def drop_columns(self, *fields: Union[str, Expression]) -> 'Table':
+    def drop_columns(self, *fields: Expression) -> 'Table':
         """
         Drops existing columns. The field expressions should be field reference expressions.
 
@@ -788,15 +758,10 @@ class Table(object):
         :param fields: Column list string.
         :return: The result table.
         """
-        if all(isinstance(f, Expression) for f in fields):
-            return Table(self._j_table.dropColumns(to_expression_jarray(fields)),
-                         self._t_env)
-        else:
-            assert len(fields) == 1
-            assert isinstance(fields[0], str)
-            return Table(self._j_table.dropColumns(fields[0]), self._t_env)
+        return Table(self._j_table.dropColumns(to_expression_jarray(fields)),
+                     self._t_env)
 
-    def map(self, func: Union[str, Expression, UserDefinedScalarFunctionWrapper]) -> 'Table':
+    def map(self, func: Union[Expression, UserDefinedScalarFunctionWrapper]) -> 'Table':
         """
         Performs a map operation with a user-defined scalar function.
 
@@ -816,15 +781,13 @@ class Table(object):
 
         .. versionadded:: 1.13.0
         """
-        if isinstance(func, str):
-            return Table(self._j_table.map(func), self._t_env)
-        elif isinstance(func, Expression):
+        if isinstance(func, Expression):
             return Table(self._j_table.map(func._j_expr), self._t_env)
         else:
             func._set_takes_row_as_input()
             return Table(self._j_table.map(func(with_columns(col("*")))._j_expr), self._t_env)
 
-    def flat_map(self, func: Union[str, Expression, UserDefinedTableFunctionWrapper]) -> 'Table':
+    def flat_map(self, func: Union[Expression, UserDefinedTableFunctionWrapper]) -> 'Table':
         """
         Performs a flatMap operation with a user-defined table function.
 
@@ -848,15 +811,13 @@ class Table(object):
 
         .. versionadded:: 1.13.0
         """
-        if isinstance(func, str):
-            return Table(self._j_table.flatMap(func), self._t_env)
-        elif isinstance(func, Expression):
+        if isinstance(func, Expression):
             return Table(self._j_table.flatMap(func._j_expr), self._t_env)
         else:
             func._set_takes_row_as_input()
             return Table(self._j_table.flatMap(func(with_columns(col("*")))._j_expr), self._t_env)
 
-    def aggregate(self, func: Union[str, Expression, UserDefinedAggregateFunctionWrapper]) \
+    def aggregate(self, func: Union[Expression, UserDefinedAggregateFunctionWrapper]) \
             -> 'AggregatedTable':
         """
         Performs a global aggregate operation with an aggregate function. You have to close the
@@ -878,16 +839,14 @@ class Table(object):
             ...                   [DataTypes.FIELD("a", DataTypes.FLOAT()),
             ...                    DataTypes.FIELD("b", DataTypes.INT())]),
             ...               func_type="pandas")
-            >>> tab.aggregate(agg.alias("a", "b")).select(col("a"), col("b"))
+            >>> tab.aggregate(agg.alias("a", "b")).select(col('a'), col('b'))
 
         :param func: user-defined aggregate function.
         :return: The result table.
 
         .. versionadded:: 1.13.0
         """
-        if isinstance(func, str):
-            return AggregatedTable(self._j_table.aggregate(func), self._t_env)
-        elif isinstance(func, Expression):
+        if isinstance(func, Expression):
             return AggregatedTable(self._j_table.aggregate(func._j_expr), self._t_env)
         else:
             func._set_takes_row_as_input()
@@ -898,7 +857,7 @@ class Table(object):
                 func = func(with_columns(col("*")))
             return AggregatedTable(self._j_table.aggregate(func._j_expr), self._t_env)
 
-    def flat_aggregate(self, func: Union[str, Expression, UserDefinedAggregateFunctionWrapper]) \
+    def flat_aggregate(self, func: Union[Expression, UserDefinedAggregateFunctionWrapper]) \
             -> 'FlatAggregateTable':
         """
         Perform a global flat_aggregate without group_by. flat_aggregate takes a
@@ -935,16 +894,14 @@ class Table(object):
             ...         return DataTypes.ROW(
             ...             [DataTypes.FIELD("a", DataTypes.BIGINT())])
             >>> top2 = udtaf(Top2())
-            >>> tab.flat_aggregate(top2.alias("a", "b")).select(col("a"), col("b"))
+            >>> tab.flat_aggregate(top2.alias("a", "b")).select(col('a'), col('b'))
 
         :param func: user-defined table aggregate function.
         :return: The result table.
 
         .. versionadded:: 1.13.0
         """
-        if isinstance(func, str):
-            return FlatAggregateTable(self._j_table.flatAggregate(func), self._t_env)
-        elif isinstance(func, Expression):
+        if isinstance(func, Expression):
             return FlatAggregateTable(self._j_table.flatAggregate(func._j_expr), self._t_env)
         else:
             func._set_takes_row_as_input()
@@ -1129,7 +1086,7 @@ class GroupedTable(object):
         self._j_table = java_table
         self._t_env = t_env
 
-    def select(self, *fields: Union[str, Expression]) -> 'Table':
+    def select(self, *fields: Expression) -> 'Table':
         """
         Performs a selection operation on a grouped table. Similar to an SQL SELECT statement.
         The field expressions can contain complex expressions and aggregations.
@@ -1142,14 +1099,9 @@ class GroupedTable(object):
         :param fields: Expression string that contains group keys and aggregate function calls.
         :return: The result table.
         """
-        if all(isinstance(f, Expression) for f in fields):
-            return Table(self._j_table.select(to_expression_jarray(fields)), self._t_env)
-        else:
-            assert len(fields) == 1
-            assert isinstance(fields[0], str)
-            return Table(self._j_table.select(fields[0]), self._t_env)
+        return Table(self._j_table.select(to_expression_jarray(fields)), self._t_env)
 
-    def aggregate(self, func: Union[str, Expression, UserDefinedAggregateFunctionWrapper]) \
+    def aggregate(self, func: Union[Expression, UserDefinedAggregateFunctionWrapper]) \
             -> 'AggregatedTable':
         """
         Performs a aggregate operation with an aggregate function. You have to close the
@@ -1179,9 +1131,7 @@ class GroupedTable(object):
 
         .. versionadded:: 1.13.0
         """
-        if isinstance(func, str):
-            return AggregatedTable(self._j_table.aggregate(func), self._t_env)
-        elif isinstance(func, Expression):
+        if isinstance(func, Expression):
             return AggregatedTable(self._j_table.aggregate(func._j_expr), self._t_env)
         else:
             func._set_takes_row_as_input()
@@ -1192,7 +1142,7 @@ class GroupedTable(object):
                 func = func(with_columns(col("*")))
             return AggregatedTable(self._j_table.aggregate(func._j_expr), self._t_env)
 
-    def flat_aggregate(self, func: Union[str, Expression, UserDefinedAggregateFunctionWrapper]) \
+    def flat_aggregate(self, func: Union[Expression, UserDefinedAggregateFunctionWrapper]) \
             -> 'FlatAggregateTable':
         """
         Performs a flat_aggregate operation on a grouped table. flat_aggregate takes a
@@ -1239,9 +1189,7 @@ class GroupedTable(object):
 
         .. versionadded:: 1.13.0
         """
-        if isinstance(func, str):
-            return FlatAggregateTable(self._j_table.flatAggregate(func), self._t_env)
-        elif isinstance(func, Expression):
+        if isinstance(func, Expression):
             return FlatAggregateTable(self._j_table.flatAggregate(func._j_expr), self._t_env)
         else:
             func._set_takes_row_as_input()
@@ -1262,7 +1210,7 @@ class GroupWindowedTable(object):
         self._j_table = java_group_windowed_table
         self._t_env = t_env
 
-    def group_by(self, *fields: Union[str, Expression]) -> 'WindowGroupedTable':
+    def group_by(self, *fields: Expression) -> 'WindowGroupedTable':
         """
         Groups the elements by a mandatory window and one or more optional grouping attributes.
         The window is specified by referring to its alias.
@@ -1288,13 +1236,8 @@ class GroupWindowedTable(object):
         :param fields: Group keys.
         :return: A window grouped table.
         """
-        if all(isinstance(f, Expression) for f in fields):
-            return WindowGroupedTable(
-                self._j_table.groupBy(to_expression_jarray(fields)), self._t_env)
-        else:
-            assert len(fields) == 1
-            assert isinstance(fields[0], str)
-            return WindowGroupedTable(self._j_table.groupBy(fields[0]), self._t_env)
+        return WindowGroupedTable(
+            self._j_table.groupBy(to_expression_jarray(fields)), self._t_env)
 
 
 class WindowGroupedTable(object):
@@ -1306,7 +1249,7 @@ class WindowGroupedTable(object):
         self._j_table = java_window_grouped_table
         self._t_env = t_env
 
-    def select(self, *fields: Union[str, Expression]) -> 'Table':
+    def select(self, *fields: Expression) -> 'Table':
         """
         Performs a selection operation on a window grouped table. Similar to an SQL SELECT
         statement.
@@ -1329,7 +1272,7 @@ class WindowGroupedTable(object):
             assert isinstance(fields[0], str)
             return Table(self._j_table.select(fields[0]), self._t_env)
 
-    def aggregate(self, func: Union[str, Expression, UserDefinedAggregateFunctionWrapper]) \
+    def aggregate(self, func: Union[Expression, UserDefinedAggregateFunctionWrapper]) \
             -> 'AggregatedTable':
         """
         Performs an aggregate operation on a window grouped table. You have to close the
@@ -1343,9 +1286,9 @@ class WindowGroupedTable(object):
             ...                   [DataTypes.FIELD("a", DataTypes.FLOAT()),
             ...                    DataTypes.FIELD("b", DataTypes.INT())]),
             ...               func_type="pandas")
-            >>> window_grouped_table.group_by(col("w")) \\
-            ...     .aggregate(agg(col('b'))) \\
-            ...     .alias("c", "d") \\
+            >>> window_grouped_table.group_by(col("w")) \
+            ...     .aggregate(agg(col('b'))) \
+            ...     .alias("c", "d") \
             ...     .select(col('c'), col('d'))
             >>> # take all the columns as inputs
             >>> # pd is a Pandas.DataFrame
@@ -1361,9 +1304,7 @@ class WindowGroupedTable(object):
 
         .. versionadded:: 1.13.0
         """
-        if isinstance(func, str):
-            return AggregatedTable(self._j_table.aggregate(func), self._t_env)
-        elif isinstance(func, Expression):
+        if isinstance(func, Expression):
             return AggregatedTable(self._j_table.aggregate(func._j_expr), self._t_env)
         else:
             func._set_takes_row_as_input()
@@ -1397,7 +1338,7 @@ class OverWindowedTable(object):
         self._j_table = java_over_windowed_table
         self._t_env = t_env
 
-    def select(self, *fields: Union[str, Expression]) -> 'Table':
+    def select(self, *fields: Expression) -> 'Table':
         """
         Performs a selection operation on a over windowed table. Similar to an SQL SELECT
         statement.
@@ -1413,12 +1354,7 @@ class OverWindowedTable(object):
         :param fields: Expression string.
         :return: The result table.
         """
-        if all(isinstance(f, Expression) for f in fields):
-            return Table(self._j_table.select(to_expression_jarray(fields)), self._t_env)
-        else:
-            assert len(fields) == 1
-            assert isinstance(fields[0], str)
-            return Table(self._j_table.select(fields[0]), self._t_env)
+        return Table(self._j_table.select(to_expression_jarray(fields)), self._t_env)
 
 
 class AggregatedTable(object):
@@ -1430,7 +1366,7 @@ class AggregatedTable(object):
         self._j_table = java_table
         self._t_env = t_env
 
-    def select(self, *fields: Union[str, Expression]) -> 'Table':
+    def select(self, *fields: Expression) -> 'Table':
         """
         Performs a selection operation after an aggregate operation. The field expressions
         cannot contain table functions and aggregations.
@@ -1456,12 +1392,7 @@ class AggregatedTable(object):
         :param fields: Expression string.
         :return: The result table.
         """
-        if all(isinstance(f, Expression) for f in fields):
-            return Table(self._j_table.select(to_expression_jarray(fields)), self._t_env)
-        else:
-            assert len(fields) == 1
-            assert isinstance(fields[0], str)
-            return Table(self._j_table.select(fields[0]), self._t_env)
+        return Table(self._j_table.select(to_expression_jarray(fields)), self._t_env)
 
 
 class FlatAggregateTable(object):
@@ -1474,7 +1405,7 @@ class FlatAggregateTable(object):
         self._j_table = java_table
         self._t_env = t_env
 
-    def select(self, *fields: Union[str, Expression]) -> 'Table':
+    def select(self, *fields: Expression) -> 'Table':
         """
         Performs a selection operation on a FlatAggregateTable. Similar to a SQL SELECT statement.
         The field expressions can contain complex expressions.
@@ -1516,9 +1447,4 @@ class FlatAggregateTable(object):
         :param fields: Expression string.
         :return: The result table.
         """
-        if all(isinstance(f, Expression) for f in fields):
-            return Table(self._j_table.select(to_expression_jarray(fields)), self._t_env)
-        else:
-            assert len(fields) == 1
-            assert isinstance(fields[0], str)
-            return Table(self._j_table.select(fields[0]), self._t_env)
+        return Table(self._j_table.select(to_expression_jarray(fields)), self._t_env)
